#define UTS_MACHINE		"arm"
#define LINUX_COMPILE_BY	"builduser"
#define LINUX_COMPILE_HOST	"arch-nspawn-8295"
#define LINUX_COMPILER		"armv7l-unknown-linux-gnueabihf-gcc (GCC) 16.1.1 20260430, GNU ld (GNU Binutils) 2.46.0"
